<?php

namespace Intergo\Shortlinks;

use Intergo\Shortlinks\Enums\ShortlinksEnum;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Redis;

abstract class AbstractShortlinksService
{
    private Redis $redis;
    private ShortlinksRepository $repository;

    /**
     * AbstractWalletService constructor.
     */
    public function __construct(Redis $redis, DB $db)
    {
        $this->redis = $redis;
        $this->repository = new ShortlinksRepository($db, $this->getConnectionName());
    }

    abstract function log($message, $params);

    /**
     * @return string
     */
    protected function getConnectionName()
    {
        return ShortlinksEnum::APP_DB_CONNECTION;
    }

    /**
     * @param $authUserId
     * @param $message
     * @param $phone
     * @return mixed|string
     */
    public function replaceUniqueShortlinks($authUserId, $message, $phone)
    {
        $uniqueShortlinks = $this->getUniqueShortlinks($authUserId, $message);
        if (!empty($uniqueShortlinks)) {
            $message = $this->replaceShortlinks($uniqueShortlinks, $message, $phone);
        }
        return $message;
    }

    /**
     * Check if unique shortlink exists
     *
     * @param $authUserId
     * @param $message
     *
     * @return array
     */
    protected function getUniqueShortlinks($authUserId, $message)
    {
        $shortlinks = $this->repository->getUserShortlinks($authUserId);
        $shortlinks = $shortlinks->filter(function ($shortlink) use ($message) {
            return ($shortlink->unique && (strpos($message, $shortlink->shortlink) !== false));
        });
        return $shortlinks->toArray();
    }

    /**
     * @param $uniqueShortlinks
     * @param $message
     * @param $phone
     * @return string
     */
    protected function replaceShortlinks($uniqueShortlinks, $message, $phone)
    {
        foreach ($uniqueShortlinks as $shortlink) {
            $newShortlink = $this->createFromParent($shortlink->id, $phone);
            $message = str_replace($shortlink->shortlink, $newShortlink, $message);
        }
        return $message;
    }

    /**
     * @param $shortlinkId
     * @param $phone
     * @return string
     */
    public function createFromParent($shortlinkId, $phone)
    {
        $newCode = $this->generateShortlink(7);
        $cacheKey = $this->getCacheKeyForChild($shortlinkId, $newCode);
        $value = ['phone' => $phone, 'parent' => $shortlinkId,];

        $this->log("Send Cycle - Prepare Unique Shortlink", ['shortlink_code' => $newCode, 'value' => $value]);

        $this->redis::connection(ShortlinksEnum::CACHE_CONNECTION)->set($cacheKey, serialize($value));
        return $newCode;
    }

    /**
     * @param $authUserId
     * @param int $totalValidMessages
     * @param string|null $message
     * @param string|null $viberTargetUrl
     * @return void
     */
    public function updateShortlinksMessageCount($authUserId, int $totalValidMessages, ?string $message, ?string $viberTargetUrl)
    {
        $shortlinks = $this->repository->getUserShortlinks($authUserId);
        foreach ($shortlinks as $shortlink) {
            if ((strpos($message, $shortlink->shortlink) !== false) || (strpos($viberTargetUrl, $shortlink->shortlink) !== false)) {
                $message_count = $shortlink->message_count + $totalValidMessages;
                $this->repository->updateShortlink($shortlink->id, compact('message_count'));
            }
        }
    }

    /**
     * @param $parentId
     * @param $shortlink
     * @return string
     */
    public function getCacheKeyForChild($parentId, $shortlink)
    {
        return "Shortlink:parent:$parentId:shortlink:$shortlink";
    }

    /**
     * @param int $length
     * @return string
     */
    public function generateShortlink(int $length = 5)
    {
        try {
            return bin2hex(random_bytes($length));
        } catch (\Throwable $exception) {
            for ($s = '', $i = 0, $z = strlen($a = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789')-1; $i != $n; $x = rand(0,$z), $s .= $a[$x], $i++);
            return $s;
        }
    }
}
